
#include <stdlib.h>
#include "types.h"
#include "CDib.h"
#include "nodes.h"
#include "tools.h"
#include "snapshots.h"
#include "resource.h"
#include "afxres.h"

#include <conio.h>

#pragma pack ( 1 )

typedef struct {
	u16 definition;
	u8	extralife_y;
	u8	extralife_x;

	struct {
		u8	fooditem : 3 ;	
		u8	fooditemavailable : 1 ;
		u8	extralife : 1 ;
		u8	dynamite : 1 ;
		u8	unused : 2 ;
	} roomflags ;
	
	u8	fooditem_y;
	u8	fooditem_x;
	u16 nasties;
	u8	dynamite_y;
	u8	dynamite_x;

	struct {
		u8	oxygen : 1 ;
		u8	creditcard : 1 ;
		u8	deoderant : 1 ;
		u8	unused	: 4 ;
		u8	visited : 1 ;
	} objflags ;

} roomhdr_t  ;

#pragma pack ()




void DisplaySprite ( int startx, int starty, u16 table[], int sprite );
void DisplaySprite ( int startx, int starty, u16 addr, int width, int height, u8 attr=7 );
void DisplaySprite ( int startx, int starty, u16 addr, u8 attr=0 );

void DisplayBlockGraphic ( int x, int y, u8* data, int width, int height, u8 attr=0 );
void DisplayPlatformSprites ( int startx, int starty, u16 addr, int total );
void DisplayAnimSprites ( int startx, int starty, u16 addr, int total );


void DisplayAnimSprite ( int startx, int starty, int sprite, int frame, u8 attr );
void DisplayPlatform ( int startx, int starty, int platform );
void DisplayFood ( int startx, int starty, int food );
void DrawRoom ( int room );


char szAppName[] = APPNAME; // The name of this application
char szTitle[]   = APPNAME": Nodes of Yesod Viewer"; // The title bar text

CDib*		pSpecScreen;
HPALETTE	hPalSpec;
u8*			specmem=NULL;
HDC			hDcMain;
int			room;
char		number[10];


void nodes_Init()
{
	InitSpectrum();
	LoadSnapshot ( "d:\\ddan\\dd.sna" );

	room = 0x00;
}

void nodes_DeInit()
{
	DeInitSpectrum();
}


void nodes_Paint ( HDC hDC )
{
RECT	rect;

	hDcMain = hDC;

	if ( !pSpecScreen ) return;

	SetRect ( &rect, 0, 0, SCREEN_WIDTH*2, SCREEN_HEIGHT*2 );
	FillRect ( hDcMain, &rect, GetStockObject(BLACK_BRUSH) );
	memset ( pSpecScreen->m_pBits,0x00, SCREEN_WIDTH*SCREEN_HEIGHT );

//	DisplayAnimSprites ( 8, 8, 0xae60, 61 );
//	DisplaySprites ( 8, 8, 0x633a, 24, 28, 8 );
//	DisplayPlatformSprites ( 8, 8, 0x6E00, 7 );
//	DisplayPlatformSprites ( 8, 8, 0xF052, 9 );
//	DisplayPlatformSprites ( 8, 8, 0x6C48, 128 );
//	DisplayPlatformSprites ( 8, 8, 0x6D48, 92 );
//	DisplayPlatformSprites ( 8, 8, 0x65da, 8 );

//	DisplaySprite ( 8, 8, 0xef5d, 7 );
//	DisplaySprite ( 16, 16, 0xef49, 7 );

//	DisplaySprite ( 8, 8, 0xebdc ); //safe
//	DisplaySprite ( 16, 16, 0xeb89 ); //plans

	DisplayPlatform ( 8,8, 0x50 );
	pSpecScreen->Draw( hDcMain, 0, 0, 0, 0, 256, 192, 2 );

//	DrawRoom(47-room);
}


void DrawRoom ( int room )
{
u8 nasties_graphic[4];
u8 nasties_count[4];

	// get the screen info
	roomhdr_t* roomhdr = (roomhdr_t*)&specmem[ 0x69e6 ];
	roomhdr += room;

	memset ( pSpecScreen->m_pBits,0x00, SCREEN_WIDTH*SCREEN_HEIGHT );

	u8* data = &specmem[ roomhdr->definition ] ;
//	u8* data = &specmem[ 0xf4fe ] ;

	int x;
	int y;
	int platform;
	int delta;
	int count;
	int ii;

	while ( data[0] != 0xff ) {
		y = *data++;
		x = *data++;
		platform = (*data++)-1;


		switch ( *data ) {
			case 0xfb:
				data++;
				delta=*data++;
				count=*data++;
				for ( ii=0; ii<count; ii++, y-=delta, x++ )
					DisplayPlatform ( x, y, platform );
				break;
			case 0xfc:
				data++;
				delta=*data++;
				count=*data++;
				for ( ii=0; ii<count; ii++, y+=delta, x++ )
					DisplayPlatform ( x, y, platform );
				break;
			case 0xfd:
				data++;
				delta=*data++;
				count=*data++;
				for ( ii=0; ii<count; ii++, y-=delta )
					DisplayPlatform ( x, y, platform );
				break;
			case 0xfe:
				data++;
				delta=*data++;
				count=*data++;
				for ( ii=0; ii<count; ii++, x+=delta )
					DisplayPlatform ( x, y, platform );
				break;
			default:
				DisplayPlatform ( x, y, platform );
				break;
		}
	}

#if 1
	//if ( roomhdr->extralife_y<20 )

	DisplaySprite ( roomhdr->extralife_x, roomhdr->extralife_y, 0xef5d, 7 );
	DisplayFood ( roomhdr->fooditem_x, roomhdr->fooditem_y, roomhdr->roomflags.fooditem );
	DisplaySprite ( roomhdr->dynamite_x, roomhdr->dynamite_y, 0xef49, 7 );

	data = (u8*) &specmem[0xf410];
	data += (room*2);
	y=data[0];
	x=data[1];
	DisplayPlatform ( x, y, 0xcb );

	
	data = (u8*) &specmem[roomhdr->nasties];
	
	for ( ii=0; ii<4; ii++ )
		nasties_graphic[ii]=*data++;

	nasties_count[0] = HINIBBLE( *data );
	nasties_count[1] = LONIBBLE( *data );
	data++;
	nasties_count[2] = HINIBBLE( *data );
	nasties_count[3] = LONIBBLE( *data );
	data++;

	for ( ii=0; ii<4; ii++ ) {
		for ( int jj=0; jj<nasties_count[ii]; jj++ ) {
			y = *data++;
			x = *data++;
			u8 min = *data++;
			u8 max = *data++;
			u8 flags = *data++;
			u8 attr = *data++;
			u8 unknown1 = *data++;
			u8 unknown2 = *data++;

			DisplayAnimSprite ( x, y, nasties_graphic[ii], 0, attr );
		}
	}


	if ( room < 8 ) {
		data = (u8*) &specmem[0x6985];
		DisplayGraphic ( 0, 20*8, data, 32, 3, 0x4f );
	}

#endif

	pSpecScreen->Draw( hDcMain, 0, 0, 0, 0, 256, 192, 2 );
	DrawGrid();

	sprintf ( number, "%02x", 47-room );
	Text ( 16, 340, number );

}




void DisplayAnimSprites ( int startx, int starty, u16 addr, int total )
{
int x,y;
int highest;

	u16* table = (u16*) &specmem[addr];

	x= startx;
	y= starty;
	highest=0;

	for ( int ii=0; ii<total; ii++ ) {

		u16 addr = table[ii];
		u8* data = &specmem [ addr ];
		u8 flag = *data++;

		int frames = 4;
		int bytewidth = flag&0x03;
		int byteheight = ((flag>>2)&0x03);
		int pixelwidth = bytewidth*8;
		int pixelheight = byteheight*8;

		addr++;

		for ( int iii=0; iii<frames; iii++ ) {
			//int bytewidth = *data++;
			//int byteheight = *data++;

			if ( x+pixelwidth > 500  ) {
				y+= highest+3;
				highest=0;
				x=startx;
			}

			if ( y+pixelheight > 480 )
				break;

			DisplaySprite ( 0, 0, addr, pixelwidth, pixelheight );
			pSpecScreen->Draw( hDcMain, x, y, 0, 0, pixelwidth, pixelheight );
			DrawRectangle ( x-1, y-1, pixelwidth+2, pixelheight+2, RGB(255,0,255) );

			if ( pixelheight > highest ) highest=pixelheight;
			x+= (pixelwidth)+3;
			addr+=bytewidth*pixelheight;
		}

	}
}


void DisplayPlatform ( int startx, int starty, int platform )
{
	u16* table = (u16*) &specmem[0x6C48];
	DisplaySprite ( startx, starty, table[platform] );

}

void DisplayFood ( int startx, int starty, int food )
{
	u16* table = (u16*) &specmem[0x65da];
	DisplaySprite ( startx, starty, table[food] );

}


void DisplayAnimSprite ( int startx, int starty, int sprite, int frame, u8 attr )
{
	u16* table = (u16*) &specmem[0xae60];

	u16 addr = table[sprite];

	u8* data = &specmem [ addr ];

	u8 flag = *data++;

	addr++;

	int bytewidth = flag&0x03;
	int byteheight = ((flag>>2)&0x03);
	int pixelwidth = bytewidth*8;
	int pixelheight = byteheight*8;

	DisplaySprite ( startx*8, starty*8, addr, pixelwidth, pixelheight, attr );
}


void DisplayPlatformSprites ( int startx, int starty, u16 addr, int total )
{
int x,y;
int highest;

	u16* table = (u16*) &specmem[addr];

	x= startx;
	y= starty;
	highest=0;

	for ( int ii=0; ii<total; ii++ ) {

		u8* data = &specmem [ table[ii] ];

		int bytewidth = *data++;
		int byteheight = *data++;
		int pixelwidth = bytewidth*8;
		int pixelheight = byteheight*8;

		if ( x+pixelwidth > 500  ) {
			y+= highest+3;
			highest=0;
			x=startx;
		}

		if ( y+pixelheight > 480 )
			break;

		DisplayBlockGraphic ( 0, 0, data, bytewidth, byteheight );
		pSpecScreen->Draw( hDcMain, x, y, 0, 0, pixelwidth, pixelheight );
		DrawRectangle ( x-1, y-1, pixelwidth+2, pixelheight+2, RGB(255,0,255) );

		if ( pixelheight > highest ) highest=pixelheight;
		x+= (pixelwidth)+3;
	}
}



void DisplaySprite ( int startx, int starty, u16 table[], int sprite )
{
int pixelwidth,bytewidth, height;
u8	attr;

	u8* data = &specmem [ table[sprite] ];
	pixelwidth = *data++;
	height = *data++;
	attr = *data++;
	bytewidth = (pixelwidth/8);
	if ( pixelwidth%8 ) bytewidth++;
	DisplayGraphic ( startx, starty, data, bytewidth, height, attr );

}

void DisplaySprite ( int startx, int starty, u16 addr, u8 attr )
{
	u8* data = &specmem [ addr ];
	int bytewidth = *data++;
	int byteheight = *data++;
	int pixelwidth = bytewidth*8;
	int pixelheight = byteheight*8;
	DisplayBlockGraphic ( startx, starty, data, bytewidth, byteheight, attr );

}


void DisplayBlockGraphic ( int x, int y, u8* data, int width, int height, u8 attr2 )
{
u8* attr = data + ( width*height*8) ;


	u8 attrib=0;
	if ( attr[0] == 0 )
		attrib=attr[1];

	if ( attr2 ) attrib=attr2;

	y*=8;

	int y2=y; //+((height-1)*8);
	for ( int y1=0; y1<height; y1++ ) {
		for ( int y3=0; y3<8; y3++ ) {
			int x2=x;
			for ( int x1=0; x1<width; x1++ ) {
				if ( attrib )
					PlotByte ( x2*8, y2, *data++, attrib );
				else
				PlotByte ( x2*8, y2, *data++, attr[x1] );
				x2++;
			}
			y2++;
		}
		attr+=width;
		y2-=16;
	}


}




void nodes_OnCommand(HWND hWnd, int id, HWND hWndCtl, UINT codeNotify)
{
char buffer[_MAX_PATH];

	switch ( id ) {

		case ID_APP_EXIT:
			DestroyWindow (hWnd);
			break;

		case ID_FILE_SAVE:
			sprintf ( buffer, "\\%02x.bmp", room );
			pSpecScreen->Save(buffer);
			break;

		case ID_RIGHT:
			if ( (room%8) < 7 ) {
				room++;
				InvalidateRect(hWnd,NULL,FALSE);
			}
			break;
		case ID_LEFT:
			if ( (room%8) > 0 ) {
				room--;		
				InvalidateRect(hWnd,NULL,FALSE);
			}
			break;
		case ID_DOWN:
			if ( room<40 ) {
				room+=8;
				InvalidateRect(hWnd,NULL,FALSE);
			}
			break;
		case ID_UP:
			if ( room >7 ) {
				room-=8;
				InvalidateRect(hWnd,NULL,FALSE);
			}
			break;

		default:
			FORWARD_WM_COMMAND (hWnd, id, hWndCtl, codeNotify, DefWindowProc );
	}

}

